@@ -15,46 +15,58 @@
 # You should have received a copy of the GNU General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
 # Street, Fifth Floor, Boston, MA 02110-1301 USA.
-from __future__ import print_function, division
+from __future__ import division, print_function
 
-import esptool
 import argparse
-import sys
 import os
 import struct
+import sys
 import time
 
+import esptool
+
 # Table of efuse values - (category, block, word in block, mask, write disable bit, read disable bit, register name, type, description)
 # Match values in efuse_reg.h & Efuse technical reference chapter
 EFUSES = [
-    ('WR_DIS',               "efuse",    0, 0, 0x0000FFFF, 1, None, "int", "Efuse write disable mask"),
-    ('RD_DIS',               "efuse",    0, 0, 0x000F0000, 0, None, "int", "Efuse read disablemask"),
-    ('FLASH_CRYPT_CNT',      "security", 0, 0, 0x0FF00000, 2, None, "bitcount", "Flash encryption mode counter"),
-    ('MAC',                  "identity", 0, 1, 0xFFFFFFFF, 3, None, "mac", "MAC Address"),
-    ('XPD_SDIO_FORCE',       "config",   0, 4, 1 << 16,    5, None, "flag", "Ignore MTDI pin (GPIO12) for VDD_SDIO on reset"),
-    ('XPD_SDIO_REG',         "config",   0, 4, 1 << 14,    5, None, "flag", "If XPD_SDIO_FORCE, enable VDD_SDIO reg on reset"),
-    ('XPD_SDIO_TIEH',        "config",   0, 4, 1 << 15,    5, None, "flag", "If XPD_SDIO_FORCE & XPD_SDIO_REG, 1=3.3V 0=1.8V"),
-    ('SPI_PAD_CONFIG_CLK',   "config",   0, 5, 0x1F << 0,  6, None, "spipin", "Override SD_CLK pad (GPIO6/SPICLK)"),
-    ('SPI_PAD_CONFIG_Q',     "config",   0, 5, 0x1F << 5,  6, None, "spipin", "Override SD_DATA_0 pad (GPIO7/SPIQ)"),
-    ('SPI_PAD_CONFIG_D',     "config",   0, 5, 0x1F << 10, 6, None, "spipin", "Override SD_DATA_1 pad (GPIO8/SPID)"),
-    ('SPI_PAD_CONFIG_HD',    "config",   0, 3, 0x1F << 4,  3, None, "spipin", "Override SD_DATA_2 pad (GPIO9/SPIHD)"),
-    ('SPI_PAD_CONFIG_CS0',   "config",   0, 5, 0x1F << 15, 6, None, "spipin", "Override SD_CMD pad (GPIO11/SPICS0)"),
-    ('FLASH_CRYPT_CONFIG',   "security", 0, 5, 0x0F << 28, 10, 3, "int", "Flash encryption config (key tweak bits)"),
-    ('CHIP_VERSION',         "identity", 0, 3, 0x0F << 12, 0,  0, "int", "Chip version"),
-    ('CHIP_PACKAGE',         "identity", 0, 3, 0x07 << 9,  0,  0, "int", "Chip package identifier"),
-    ('CODING_SCHEME',        "efuse",    0, 6, 0x3,        10, 3, "int", "Efuse variable block length scheme"),
-    ('CONSOLE_DEBUG_DISABLE',"security", 0, 6, 1 << 2,     15, None, "flag", "Disable ROM BASIC interpreter fallback"),
-    ('DISABLE_SDIO_HOST',    "config",   0, 6, 1 << 3,     None, None, "flag", "Disable SDIO host"),
-    ('ABS_DONE_0',           "security", 0, 6, 1 << 4,     12, None, "flag", "secure boot enabled for bootloader"),
-    ('ABS_DONE_1',           "security", 0, 6, 1 << 5,     13, None, "flag", "secure boot abstract 1 locked"),
-    ('JTAG_DISABLE',         "security", 0, 6, 1 << 6,     14, None, "flag", "Disable JTAG"),
-    ('DISABLE_DL_ENCRYPT',   "security", 0, 6, 1 << 7,     15, None, "flag", "Disable flash encryption in UART bootloader"),
-    ('DISABLE_DL_DECRYPT',   "security", 0, 6, 1 << 8,     15, None, "flag", "Disable flash decryption in UART bootloader"),
-    ('DISABLE_DL_CACHE',     "security", 0, 6, 1 << 9,     15, None, "flag", "Disable flash cache in UART bootloader"),
-    ('KEY_STATUS',           "efuse",    0, 6, 1 << 10,    10, 3, "flag", "Usage of efuse block 3 (reserved)"),
-    ('BLK1',                 "security", 1, 0, 0xFFFFFFFF, 7,  0, "keyblock", "Flash encryption key"),
-    ('BLK2',                 "security", 2, 0, 0xFFFFFFFF, 8,  1, "keyblock", "Secure boot key"),
-    ('BLK3',                 "security", 3, 0, 0xFFFFFFFF, 9,  2, "keyblock", "Variable Block 3"),
+    ('WR_DIS',               "efuse",       0, 0, 0x0000FFFF, 1, None, "int", "Efuse write disable mask"),
+    ('RD_DIS',               "efuse",       0, 0, 0x000F0000, 0, None, "int", "Efuse read disablemask"),
+    ('FLASH_CRYPT_CNT',      "security",    0, 0, 0x0FF00000, 2, None, "bitcount", "Flash encryption mode counter"),
+    ('MAC',                  "identity",    0, 1, 0xFFFFFFFF, 3, None, "mac", "MAC Address"),
+    ('XPD_SDIO_FORCE',       "config",      0, 4, 1 << 16,    5, None, "flag", "Ignore MTDI pin (GPIO12) for VDD_SDIO on reset"),
+    ('XPD_SDIO_REG',         "config",      0, 4, 1 << 14,    5, None, "flag", "If XPD_SDIO_FORCE, enable VDD_SDIO reg on reset"),
+    ('XPD_SDIO_TIEH',        "config",      0, 4, 1 << 15,    5, None, "flag", "If XPD_SDIO_FORCE & XPD_SDIO_REG, 1=3.3V 0=1.8V"),
+    ('SPI_PAD_CONFIG_CLK',   "config",      0, 5, 0x1F << 0,  6, None, "spipin", "Override SD_CLK pad (GPIO6/SPICLK)"),
+    ('SPI_PAD_CONFIG_Q',     "config",      0, 5, 0x1F << 5,  6, None, "spipin", "Override SD_DATA_0 pad (GPIO7/SPIQ)"),
+    ('SPI_PAD_CONFIG_D',     "config",      0, 5, 0x1F << 10, 6, None, "spipin", "Override SD_DATA_1 pad (GPIO8/SPID)"),
+    ('SPI_PAD_CONFIG_HD',    "config",      0, 3, 0x1F << 4,  3, None, "spipin", "Override SD_DATA_2 pad (GPIO9/SPIHD)"),
+    ('SPI_PAD_CONFIG_CS0',   "config",      0, 5, 0x1F << 15, 6, None, "spipin", "Override SD_CMD pad (GPIO11/SPICS0)"),
+    ('FLASH_CRYPT_CONFIG',   "security",    0, 5, 0x0F << 28, 10, 3, "int", "Flash encryption config (key tweak bits)"),
+    ('CHIP_VER_REV1',        "identity",    0, 3, 1 << 15,    0,  0, "flag", "Silicon Revision 1"),
+    ('BLK3_PART_RESERVE',    "calibration", 0, 3, 1 << 14,    0,  0, "flag", "BLOCK3 partially served for ADC calibration data"),
+    ('CHIP_VERSION',         "identity",    0, 3, 0x03 << 12, 0,  0, "int", "Reserved for future chip versions"),
+    ('CHIP_PACKAGE',         "identity",    0, 3, 0x07 << 9,  0,  0, "int", "Chip package identifier"),
+    ('CODING_SCHEME',        "efuse",       0, 6, 0x3,        10, 3, "int", "Efuse variable block length scheme"),
+    ('CONSOLE_DEBUG_DISABLE',"security",    0, 6, 1 << 2,     15, None, "flag", "Disable ROM BASIC interpreter fallback"),
+    ('DISABLE_SDIO_HOST',    "config",      0, 6, 1 << 3,     None, None, "flag", "Disable SDIO host"),
+    ('ABS_DONE_0',           "security",    0, 6, 1 << 4,     12, None, "flag", "secure boot enabled for bootloader"),
+    ('ABS_DONE_1',           "security",    0, 6, 1 << 5,     13, None, "flag", "secure boot abstract 1 locked"),
+    ('JTAG_DISABLE',         "security",    0, 6, 1 << 6,     14, None, "flag", "Disable JTAG"),
+    ('DISABLE_DL_ENCRYPT',   "security",    0, 6, 1 << 7,     15, None, "flag", "Disable flash encryption in UART bootloader"),
+    ('DISABLE_DL_DECRYPT',   "security",    0, 6, 1 << 8,     15, None, "flag", "Disable flash decryption in UART bootloader"),
+    ('DISABLE_DL_CACHE',     "security",    0, 6, 1 << 9,     15, None, "flag", "Disable flash cache in UART bootloader"),
+    ('KEY_STATUS',           "efuse",       0, 6, 1 << 10,    10, 3, "flag", "Usage of efuse block 3 (reserved)"),
+    ('ADC_VREF',             "calibration", 0, 4, 0x1F << 8,  0,  0, "vref", "Voltage reference calibration"),
+    ('BLK1',                 "security",    1, 0, 0xFFFFFFFF, 7,  0, "keyblock", "Flash encryption key"),
+    ('BLK2',                 "security",    2, 0, 0xFFFFFFFF, 8,  1, "keyblock", "Secure boot key"),
+    ('BLK3',                 "security",    3, 0, 0xFFFFFFFF, 9,  2, "keyblock", "Variable Block 3"),
+]
+
+# if BLK3_PART_RESERVE is set, these efuse fields are in BLK3:
+BLK3_PART_EFUSES = [
+    ('ADC1_TP_LOW',  "calibration", 3, 3, 0x7F << 0,   9, 2, "adc_tp", "ADC1 150mV reading"),
+    ('ADC1_TP_HIGH', "calibration", 3, 3, 0x1FF << 7,  9, 2, "adc_tp", "ADC1 850mV reading"),
+    ('ADC2_TP_LOW',  "calibration", 3, 3, 0x7F << 16,  9, 2, "adc_tp", "ADC2 150mV reading"),
+    ('ADC2_TP_HIGH', "calibration", 3, 3, 0x1FF << 23, 9, 2, "adc_tp", "ADC2 850mV reading"),
 ]
 
 # Offsets and lengths of each of the 4 efuse blocks
@@ -126,6 +138,8 @@ class EfuseField(object):
             "mac": EfuseMacField,
             "keyblock": EfuseKeyblockField,
             "spipin": EfuseSpiPinField,
+            "vref": EfuseVRefField,
+            "adc_tp": EfuseAdcPointCalibration,
         }.get(category, EfuseField)(esp, *efuse_tuple)
 
     def __init__(self, esp, register_name, category, block, word, mask, write_disable_bit, read_disable_bit, efuse_type, description):
@@ -200,14 +214,42 @@ class EfuseMacField(EfuseField):
         words = [self.esp.read_efuse(self.data_reg_offs + word) for word in [1,0]]
         # endian-swap into a bitstring
         bitstring = struct.pack(">II", *words)
-        return bitstring[2:]  # trim 2 byte CRC from the beginning (currently doesn't verify)
+        return bitstring[2:]  # trim 2 byte CRC from the beginning
 
     def get(self):
-        return hexify(self.get_raw(), ":")
+        stored_crc = self.get_stored_crc()
+        calc_crc = self.calc_crc()
+        if calc_crc == stored_crc:
+            valid_msg = "(CRC %02x OK)" % stored_crc
+        else:
+            valid_msg = "(CRC %02x invalid - calculated 0x%02x)" % (stored_crc, calc_crc)
+        return "%s %s" % (hexify(self.get_raw(), ":"), valid_msg)
 
     def burn(self, new_value):
-        # need to calculate the CRC before we can write the MAC
-        raise esptool.FatalError("Writing MAC address is not yet supported")
+        # Writing the BLK0 default MAC is not sensible, as it's written in the factory.
+        #
+        # TODO: support writing a new base MAC @ efuse BLK3
+        raise esptool.FatalError("Writing MAC address is not supported")
+
+    def get_stored_crc(self):
+        return (self.esp.read_efuse(self.data_reg_offs + 1) >> 16) & 0xFF
+
+    def calc_crc(self):
+        """
+        This algorithm is the equivalent of esp_crc8() in ESP32 ROM code
+
+        This is CRC-8 w/ inverted polynomial value 0x8C & initial value 0x00.
+        """
+        mac = self.get_raw()
+        result = 0x00
+        for b in struct.unpack("B" * 6, mac):
+            result ^= b
+            for _ in range(8):
+                lsb = result & 1
+                result >>= 1
+                if lsb != 0:
+                    result ^= 0x8c
+        return result
 
 
 class EfuseKeyblockField(EfuseField):
@@ -250,6 +292,45 @@ class EfuseSpiPinField(EfuseField):
         return super(EfuseSpiPinField, self).burn(new_value)
 
 
+class EfuseVRefField(EfuseField):
+    VREF_OFFSET = 1100  # ideal efuse value in mV
+    VREF_STEP_SIZE = 7  # 1 count in efuse == 7mV
+    VREF_SIGN_BIT = 0x10
+    VREF_MAG_BITS = 0x0F
+
+    def get(self):
+        val = self.get_raw()
+        # sign-magnitude format
+        if (val & self.VREF_SIGN_BIT):
+            val = -(val & self.VREF_MAG_BITS)
+        else:
+            val = (val & self.VREF_MAG_BITS)
+        val *= self.VREF_STEP_SIZE
+        return self.VREF_OFFSET + val
+
+    def burn(self, new_value):
+        raise RuntimeError("Writing to VRef is not supported.")
+
+
+class EfuseAdcPointCalibration(EfuseField):
+    TP_OFFSET = {  # See TP_xxxx_OFFSET in esp_adc_cal.c in ESP-IDF
+        "ADC1_TP_LOW":  278,
+        "ADC2_TP_LOW":  421,
+        "ADC1_TP_HIGH": 3265,
+        "ADC2_TP_HIGH": 3406,
+    }
+    SIGN_BIT = (0x40, 0x100)  # LOW, HIGH (2s complement format)
+    STEP_SIZE = 4
+
+    def get(self):
+        idx = 0 if self.register_name.endswith("LOW") else 1
+        sign_bit = self.SIGN_BIT[idx]
+        offset = self.TP_OFFSET[self.register_name]
+        raw = self.get_raw()
+        delta = (raw & (sign_bit - 1)) - (raw & sign_bit)
+        return offset + (delta * self.STEP_SIZE)
+
+
 def dump(esp, _efuses, args):
     """ Dump raw efuse data registers """
     for block in range(len(EFUSE_BLOCK_OFFS)):
@@ -266,7 +347,7 @@ def summary(esp, efuses, args):
             raw = e.get_raw()
             try:
                 raw = "(0x%x)" % raw
-            except:
+            except TypeError:
                 raw = ""
             (readable, writeable) = (e.is_readable(), e.is_writeable())
             if readable and writeable:
@@ -340,7 +421,7 @@ def read_protect_efuse(esp, efuses, args):
         efuse.disable_read()
 
 
-def write_protect_efuse(esp, efuses,args):
+def write_protect_efuse(esp, efuses, args):
     efuse = _get_efuse(efuses, args.efuse_name)
     if not efuse.is_writeable():
         print("Efuse %s is already write protected" % efuse.register_name)
@@ -363,6 +444,11 @@ def burn_key(esp, efuses, args):
     else:
         raise RuntimeError("args.block argument not in list!")
 
+    # check coding scheme
+    coding_scheme = _get_efuse(efuses, "CODING_SCHEME").get()
+    if coding_scheme != 0:
+        raise esptool.FatalError("Burning key efuses is not yet supported for 3/4 Coding Scheme")
+
     # check keyfile
     keyfile = args.keyfile
     keyfile.seek(0,2)  # seek t oend
@@ -447,6 +533,24 @@ The following efuses are burned: XPD_SDIO_FORCE, XPD_SDIO_REG, XPD_SDIO_TIEH.
     print("VDD_SDIO setting complete.")
 
 
+def adc_info(esp, efuses, args):
+    adc_vref = _get_efuse(efuses, "ADC_VREF")
+    blk3_reserve = _get_efuse(efuses, "BLK3_PART_RESERVE")
+
+    vref_raw = adc_vref.get_raw()
+    if vref_raw == 0:
+        print("ADC VRef calibration: None (1100mV nominal)")
+    else:
+        print("ADC VRef calibration: %dmV" % adc_vref.get())
+
+    if blk3_reserve.get():
+        print("ADC readings stored in efuse BLK3:")
+        print("    ADC1 Low reading  (150mV): %d" % _get_efuse(efuses, "ADC1_TP_LOW").get())
+        print("    ADC1 High reading (850mV): %d" % _get_efuse(efuses, "ADC1_TP_HIGH").get())
+        print("    ADC2 Low reading  (150mV): %d" % _get_efuse(efuses, "ADC2_TP_LOW").get())
+        print("    ADC2 High reading (850mV): %d" % _get_efuse(efuses, "ADC2_TP_HIGH").get())
+
+
 def hexify(bitstring, separator):
     try:
         as_bytes = tuple(ord(b) for b in bitstring)
@@ -455,9 +559,19 @@ def hexify(bitstring, separator):
     return separator.join(("%02x" % b) for b in as_bytes)
 
 
+def arg_auto_int(x):
+    return int(x, 0)
+
+
 def main():
     parser = argparse.ArgumentParser(description='espefuse.py v%s - ESP32 efuse get/set tool' % esptool.__version__, prog='espefuse')
 
+    parser.add_argument(
+        '--baud', '-b',
+        help='Serial port baud rate used when flashing/reading',
+        type=arg_auto_int,
+        default=os.environ.get('ESPTOOL_BAUD', esptool.ESPLoader.ESP_ROM_BAUD))
+
     parser.add_argument(
         '--port', '-p',
         help='Serial port device',
@@ -466,7 +580,7 @@ def main():
     parser.add_argument(
         '--before',
         help='What to do before connecting to the chip',
-        choices=['default_reset', 'no_reset', 'esp32r1'],
+        choices=['default_reset', 'no_reset', 'esp32r1', 'no_reset_no_sync'],
         default='default_reset')
 
     parser.add_argument('--do-not-confirm',
@@ -512,6 +626,9 @@ def main():
     p.add_argument('voltage', help='Voltage selection',
                    choices=['1.8V', '3.3V', 'OFF'])
 
+    p = subparsers.add_parser('adc_info',
+                              help='Display information about ADC calibration data stored in efuse.')
+
     args = parser.parse_args()
     print('espefuse.py v%s' % esptool.__version__)
     if args.operation is None:
@@ -521,11 +638,14 @@ def main():
     # each 'operation' is a module-level function of the same name
     operation_func = globals()[args.operation]
 
-    esp = esptool.ESP32ROM(args.port)
+    esp = esptool.ESP32ROM(args.port, baud=args.baud)
     esp.connect(args.before)
 
     # dict mapping register name to its efuse object
     efuses = [EfuseField.from_tuple(esp, efuse) for efuse in EFUSES]
+    if _get_efuse(efuses, "BLK3_PART_RESERVE").get():
+        # add these BLK3 efuses, if the BLK3_PART_RESERVE flag is set...
+        efuses += [EfuseField.from_tuple(esp, efuse) for efuse in BLK3_PART_EFUSES]
     operation_func(esp, efuses, args)
 
 
